#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from unittest import TestCase
import os
from tempfile import NamedTemporaryFile
from tempfile import mkdtemp
from shutil import rmtree

from pydenji.resourceloader import ReadResource
from pydenji.resourceloader import OverwritingWriteResource
from pydenji.resourceloader import AppendingWriteResource
from pydenji.resourceloader import NewFileWriteResource
from pydenji.resourceloader import ResourceAccessError
from pydenji.resourceloader import ExecutableResource
from pydenji.resourceloader import enumerateResources
from pydenji.resourceloader import RWResource

class MockFile(object):
    def read(self):
        return self

class TestRWResource(TestCase):
    def test_file_does_not_get_opened_if_lazy_attr_called(self):
        def opener(*args, **kwargs):
            self.fail("opener function was unexpectedly called.")
        resource = RWResource("file:///tmp/foo", "w", -1, opener)
        resource.mode
        resource.name

    def test_file_does_get_opened_if_not_lazy_attr_called(self):
        mockfile = MockFile()
        def opener(*args, **kwargs):
            return mockfile
            
        resource = RWResource("file:///tmp/foo", "w", -1, opener)
        self.assert_(mockfile is resource.read())

class TestResourceLoader(TestCase):
    # think: existing == accessible, or not?
    def setUp(self):
        temp = NamedTemporaryFile()
        temp.write("hello")
        temp.flush()
        self.temp = temp

    def tearDown(self):
        self.temp.close()
            
    def test_reading_loaded_resource(self):
        # what would I like for this?
        # - since it's a ReadResource, it should raise an exception if it does not
        # exists or if we're unable to read it because of permission issues.
        # - I should be able to specify additional data for the stream(), e.g. b and +
        # the interface can be limited to stream() & filename

        content = ReadResource("file://" + self.temp.name).read()
        self.assertEquals("hello", content)

    def test_error_not_existing_resource(self):
        # what would I like for this?
        # - since it's a ReadResource, it should raise an exception if it does not
        # exists or if we're unable to read it because of permission issues.
        # - I should be able to specify additional data for the stream(), e.g. b and +
        # the interface can be limited to stream() & filename

        self.assertRaises(IOError,
            ReadResource, "file://" + "/dmfsdmfdksm/kmfdskmfksdmfksdmfdksm")

    def test_error_unaccessible_resource(self):
        os.chmod(self.temp.name, 0)
        self.assertRaises(ResourceAccessError,
            ReadResource, "file://" + self.temp.name)


class TestWriteResource(TestCase):
    def setUp(self):
        self.tempdir = mkdtemp()

    def tearDown(self):
        rmtree(self.tempdir)

    def test_if_file_exists_but_is_not_writeable_error_raised(self):
        f = open(self.tempdir + os.sep + "newfile", "w")
        f.close()
        os.chmod(self.tempdir + os.sep + "newfile", 0)
        self.assertRaises(ResourceAccessError, OverwritingWriteResource, "file://" + self.tempdir + os.sep + "newfile")
        
    def test_overwriting_resource_allows_writing_if_file_does_not_exist(self):
        resource = OverwritingWriteResource("file://" + self.tempdir + os.sep + "newfile")

    def test_overwriting_resource_allows_writing_if_file_exists(self):
        f = open(self.tempdir + os.sep + "newfile", "w")
        f.write("abc")
        f.flush()
        resource = OverwritingWriteResource("file://" + self.tempdir + os.sep + "newfile")

    def test_overwriting_allows_writing_to_resource(self):
        resource = OverwritingWriteResource("file://" + self.tempdir + os.sep + "newfile")
        stream = resource
        stream.write("abc")
        stream.close()

        f = open(self.tempdir + os.sep + "newfile")
        self.assertEquals("abc", f.read())

    def test_appending_writing_resource_appends_if_file_exists(self):
        f = open(self.tempdir + os.sep + "newfile", "w")
        f.write("asd")
        f.close()

        resource = AppendingWriteResource("file://" + self.tempdir + os.sep + "newfile")
        stream = resource
        stream.write("fgh")
        stream.close()
        
        f = open(self.tempdir + os.sep + "newfile", "r")
        self.assertEquals("asdfgh", f.read())

    def test_appending_writing_resource_creates_if_file_does_not_exist(self):
        resource = AppendingWriteResource("file://" + self.tempdir + os.sep + "newfile")
        stream = resource
        stream.write("fgh")
        stream.close()

        f = open(self.tempdir + os.sep + "newfile", "r")
        self.assertEquals("fgh", f.read())

    def test_newfile_write_resource_creates_if_file_does_not_exist(self):
        resource = NewFileWriteResource("file://" + self.tempdir + os.sep + "newfile")

    def test_newfile_write_resource_raises_error_if_file_exists(self):
        f = open(self.tempdir + os.sep + "newfile", "w")
        f.write("asd")
        f.close()

        self.assertRaises(ResourceAccessError, NewFileWriteResource, "file://" + self.tempdir + os.sep + "newfile")

class TestExecutableResource(TestCase):
    def setUp(self):
        temp = NamedTemporaryFile()
        temp.write("hello")
        temp.flush()
        
        self.temp = temp

    def tearDown(self):
        self.temp.close()

    def test_executable_resource_can_be_loaded(self):
        os.chmod(self.temp.name, 100)
        res = ExecutableResource(self.temp.name)

    def test_not_executable_resource_raies_error(self):
        os.chmod(self.temp.name, 000)
        self.assertRaises(ResourceAccessError, ExecutableResource, self.temp.name)

class TestResourceEnumerator(TestCase):
    def setUp(self):
        self.tempdir = mkdtemp()
        f = open(self.tempdir + os.sep + "a", "w")
        f = open(self.tempdir + os.sep + "b", "w")
        f = open(self.tempdir + os.sep + "c", "w")

    def tearDown(self):
        rmtree(self.tempdir)


    def test_enumerator_returns_resources_from_a_dir(self):
        rs = enumerateResources(ReadResource(self.tempdir))
        self.assertEquals(3, len(rs))

    def test_enumerator_returns_required_type_of_resources(self):
        rs = enumerateResources(ReadResource(self.tempdir), ReadResource)
        for elem in rs:
            self.assert_(isinstance(elem, ReadResource))

    def test_enumerator_returns_same_filename_resource_if_file_passed(self):
        rs = enumerateResources(ReadResource(self.tempdir + os.sep + "a"))
        self.assertEquals(self.tempdir + os.sep + "a", rs[0].name)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni


import sys
from importlib import import_module

def import_or_reload(module_name):
    if module_name in sys.modules:
        reload(sys.modules[module_name])
    else:
        import_module(module_name)
    return sys.modules[module_name]


def get_by_fqdn(object_full_path, _module_import=import_or_reload):
    """
    @param object_full_path: string like package.module:Class.function
    While this is not as powerful as namedAny, it allows for reloading.
    """
    module_name, factory_name = object_full_path.split(":", 1)
    module = _module_import(module_name)

    latest_part = module
    for part_name in factory_name.split("."):
        latest_part = getattr(latest_part, part_name)
    return latest_part


# direct import from the Twisted project http://www.twistedmatrix.com
class _NoModuleFound(Exception):
    """
    No module was found because none exists.
    """

class InvalidName(ValueError):
    """
    The given name is not a dot-separated list of Python objects.
    """

class ModuleNotFound(InvalidName):
    """
    The module associated with the given name doesn't exist and it can't be
    imported.
    """

class ObjectNotFound(InvalidName):
    """
    The object associated with the given name doesn't exist and it can't be
    imported.
    """

def _importAndCheckStack(importName):
    """
    Import the given name as a module, then walk the stack to determine whether
    the failure was the module not existing, or some code in the module (for
    example a dependent import) failing.  This can be helpful to determine
    whether any actual application code was run.  For example, to distiguish
    administrative error (entering the wrong module name), from programmer
    error (writing buggy code in a module that fails to import).

    @raise Exception: if something bad happens.  This can be any type of
    exception, since nobody knows what loading some arbitrary code might do.

    @raise _NoModuleFound: if no module was found.
    """
    try:
        try:
            return __import__(importName)
        except ImportError:
            excType, excValue, excTraceback = sys.exc_info()
            while excTraceback:
                execName = excTraceback.tb_frame.f_globals["__name__"]
                if (execName is None or # python 2.4+, post-cleanup
                    execName == importName): # python 2.3, no cleanup
                    raise excType, excValue, excTraceback
                excTraceback = excTraceback.tb_next
            raise _NoModuleFound()
    except:
        # Necessary for cleaning up modules in 2.3.
        sys.modules.pop(importName, None)
        raise

def namedAny(name):
    """
    Retrieve a Python object by its fully qualified name from the global Python
    module namespace.  The first part of the name, that describes a module,
    will be discovered and imported.  Each subsequent part of the name is
    treated as the name of an attribute of the object specified by all of the
    name which came before it.  For example, the fully-qualified name of this
    object is 'twisted.python.reflect.namedAny'.

    @type name: L{str}
    @param name: The name of the object to return.

    @raise InvalidName: If the name is an empty string, starts or ends with
        a '.', or is otherwise syntactically incorrect.

    @raise ModuleNotFound: If the name is syntactically correct but the
        module it specifies cannot be imported because it does not appear to
        exist.

    @raise ObjectNotFound: If the name is syntactically correct, includes at
        least one '.', but the module it specifies cannot be imported because
        it does not appear to exist.

    @raise AttributeError: If an attribute of an object along the way cannot be
        accessed, or a module along the way is not found.

    @return: the Python object identified by 'name'.
    """
    if not name:
        raise InvalidName('Empty module name')

    names = name.split('.')

    # if the name starts or ends with a '.' or contains '..', the __import__
    # will raise an 'Empty module name' error. This will provide a better error
    # message.
    if '' in names:
        raise InvalidName(
            "name must be a string giving a '.'-separated list of Python "
            "identifiers, not %r" % (name,))

    topLevelPackage = None
    moduleNames = names[:]
    while not topLevelPackage:
        if moduleNames:
            trialname = '.'.join(moduleNames)
            try:
                topLevelPackage = _importAndCheckStack(trialname)
            except _NoModuleFound:
                moduleNames.pop()
        else:
            if len(names) == 1:
                raise ModuleNotFound("No module named %r" % (name,))
            else:
                raise ObjectNotFound('%r does not name an object' % (name,))

    obj = topLevelPackage
    for n in names[1:]:
        obj = getattr(obj, n)

    return obj

NI = namedAny # quick aliasing, stands for Name Importer



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from pydenji.userproperties.codescraper import get_getitem_accesses

class ArbitraryObject(object):
    def somefunc(self):
        return self.attr["somekey"]

    def otherfunc(self):
        p = self.attr["otherkey"]

class MemorizingAttrObject(object):
    def somefunc(self):
        some = self.attr
        return some["somekey"]

class TestCodescraper(unittest.TestCase):

    def test_get_getitem_accesses_returns_used_self_attr_keys(self):
        self.assertEquals(set(["somekey", "otherkey"]), get_getitem_accesses(ArbitraryObject(), "attr"))

    def test_get_getitem_accesses_returns_memorized_used_attr_keys(self):
        self.assertEquals(set(["somekey"]), get_getitem_accesses(MemorizingAttrObject(), "attr"))
        

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

"""
Basic interception package.

This is intended for internal usage only by the config package.

Evaluate different solutions (aspyct, pyaop) for everyday use.

This system uses dynamic subclassing. Original classes are untouched.
"""
# TODO: check class vs instance interception. -> are classmethods intercepted correctly?
# TODO: let interception not to depend on our context object, but use standard function args?

class _Context(object):
    def __init__(self, method, args, kwargs):
        self.method = method
        self.args = args
        self.kwargs = kwargs

    def proceed(self):
        return self.method(*self.args, **self.kwargs)


def _update_func_dict(source, dest):
    dest.__dict__.update(source.__dict__)
    return dest
    
def _interceptor_picker(original_method, method_interceptor):
    # picks the proper interceptor depending on original_method type.
    if getattr(original_method, "im_func", None):
        if getattr(original_method, "im_self"):
            return _classmethod_interceptor(original_method, method_interceptor)
        else:
            return _instancemethod_interceptor(original_method, method_interceptor)
    else:
        return _staticmethod_interceptor(original_method, method_interceptor)

def _staticmethod_interceptor(original_staticmethod, method_interceptor):
    def intercepted(*args, **kwargs):
        return method_interceptor(_Context(original_staticmethod, args, kwargs))
    return staticmethod(_update_func_dict(original_staticmethod, intercepted))

def _classmethod_interceptor(original_classmethod, method_interceptor):
    def intercepted(cls, *args, **kwargs):
        # we must not pass cls in args since classmethod will bind it.
        return method_interceptor(_Context(original_classmethod, args, kwargs))
    return classmethod(_update_func_dict(original_classmethod, intercepted))

def _instancemethod_interceptor(original_method, method_interceptor):
    def intercepted(instance, *args, **kwargs):
        return method_interceptor(_Context(original_method, (instance, ) + args, kwargs))
    return _update_func_dict(original_method, intercepted)

def intercept(cls, method_name, method_interceptor):
    # interceptor result will be returned. no auto-return of original method
    # return value will be performed.
    original_method = getattr(cls, method_name)
    intercepted = _interceptor_picker(original_method, method_interceptor)
    
    return type(cls)(cls.__name__, (cls, ), {method_name:intercepted} )
    








#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

# fetches the required args and kwargs for a function from a mapping.

from pydenji._inspect.getcallableargspec import getargspec


# important notice: every "dynamic proxy" function accepting variable positional
# and keyword arguments might disrupt wiring!

def wire(callable_obj, mapping, *call_args, **call_kwargs):
    all_arg_names, ignore, ignore, ignore = getargspec(callable_obj)

    # we might want to fetch something from our mapping to fill in
    # named arguments that were not passed, without overwriting
    # anything that was passed as a kw arg.
    for arg_name in all_arg_names[len(call_args):]:
        if arg_name in mapping:
            call_kwargs.setdefault(arg_name, mapping[arg_name])

    try:
        return callable_obj(*call_args, **call_kwargs)
    except TypeError, e:
        # improve error message rendering on wrong arg number.
        raise TypeError, "'%s' wiring error: %s" % (callable_obj, e.args[0])




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

def is_appcontext_aware(obj):
    # TODO: should we check for callability as well?
    if getattr(obj, "set_app_context", None) is not None:
        return True
    return False


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

from pydenji.config.argwiring import wire
from itertools import chain
try:
    from collections import Mapping as _BaseMappingClass
except ImportError:
    from UserDict import DictMixin as _BaseMappingClass

class NameClashError(Exception):
    pass

# TODO: make this inherit from ABC or from UserDict for 2.5
# FIXME: this makes it hard to perform proper wiring -> everything should be lazier?
# when wiring, can't distinguish between factory and instance wiring.
# we should always pass factories in config?
class PropertyAndContextMergingProxy(_BaseMappingClass):
    def __init__(self, context, mapping):
        self._context = context
        self._mapping = mapping

        self._verify_no_clashes()

    def _verify_no_clashes(self):
        common_keys = set(self._context).intersection(set(self._mapping))
        if common_keys:
            raise NameClashError, "Clashing keys between context and mapping: %s" % (
                ",".join(common_keys))

    def __len__(self):
        return len(self._context) + len(self._mapping)

    def __iter__(self):
        return chain(self._context, self._mapping)

    def __getitem__(self, key):
        if key in self._mapping:
            return self._mapping[key]
        elif key in self._context:
            # TODO: support arguments?
            return self._context.get_object(key)
        raise KeyError, "Could not find '%s' in mapping or context either"
    

class ArgNameContextWirer(object):
    def __init__(self, context, mapping):
        self.mergingproxy = PropertyAndContextMergingProxy(context, mapping)

    def wire(self, callable_obj, *args, **kwargs):
        return wire(callable_obj, self.mergingproxy, *args, **kwargs)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from pydenji.config.pythonconfig import is_object_factory, is_eager
from pydenji.appcontext.aware import is_appcontext_aware
from pydenji.config.composite import CompositeConfig

class UnconfiguredError(Exception):
    pass

class AppContext(object):
    def __init__(self, *configurations):
        conf = CompositeConfig(configurations)
        self._names_factories = self._get_all_factories(conf)
        conf.set_app_context(self)
        self._start(self._names_factories)

    def __contains__(self, key):
        return key in self._names_factories

    def __iter__(self):
        return self._names_factories.iterkeys()


    def get_object(self, name, *args, **kwargs):
        try:
            factory = self._names_factories[name]
        except KeyError:
            raise UnconfiguredError, "No factory was configured for '%s'" % name
        return self._get_instance(factory, *args, **kwargs)
    
    def _start(self, names_factories):
        for factory in names_factories.values():
            if is_eager(factory):
                self._get_instance(factory)

    def _get_instance(self, factory, *args, **kwargs):
        # this way the set_app_context() method will be called multiple times,
        # even though the object is a singleton. While it should make no harm,
        # we should think about it, might it do any harm?
        obj = factory(*args, **kwargs)
        if is_appcontext_aware(obj):
            obj.set_app_context(self)
        return obj

    @staticmethod
    def _get_all_factories(config):
        # TODO: refactor using a list comprehension/filter expr.
        names_factories = {}
        for attr in dir(config):
            value = getattr(config, attr)
            if is_object_factory(value):
                names_factories[attr] = value
        return names_factories










#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from copy import deepcopy

# TODO: this is not a FrozenDict. Should we just kill it?
class _FrozenDict(object):
    def __init__(self, d):
        self._d = deepcopy(d)

    def __getitem__(self, item):
        return self._d[item]

    def keys(self):
        return self._d.keys()

    def __contains__(self, key):
        return key in self._d

    def __iter__(self):
        return self._d.iterkeys()

    def __repr__(self):
        return "<FrozenDict(%r)>" % self._d


def UserProperties(property_mapping):
    return _FrozenDict(property_mapping)







# To change this template, choose Tools | Templates
# and open the template in the editor.

import unittest

from pydenji._aop.intercept import intercept

class TestInterception(unittest.TestCase):
    class MyTestClass(object):
        clsattr = 12

        def __init__(self, arg, clsattr=5):
            self.arg = arg
            self.clsattr = clsattr

        def method(self, value):
            return self.arg + value
        method.someattribute = 123 # used for attribute preservation test.


        def class_method(cls, value):
            return cls.clsattr + value
        class_method.someattribute = 456
        class_method = classmethod(class_method)

        def static_method(value):
            return 7 + value
        static_method.someattribute = 789
        static_method = staticmethod(static_method)
        

    # TODO: split those tests into multiple tests!
    def test_init_interception(self):
        def init_interceptor(context):
            self.assertRaises(AttributeError, getattr, context.args[0], "arg")
            context.proceed()
            instance = context.args[0]
            instance.other_arg = 5
           

        intercepted = intercept(self.MyTestClass, "__init__", init_interceptor)
        test_obj = intercepted(3)
        self.assertEquals(3, test_obj.arg)
        self.assertEquals(5, test_obj.other_arg)


    def test_instancemethod_interception(self):
        def method_interceptor(context):
            value = context.proceed()
            return value * 4

        intercepted = intercept(self.MyTestClass, "method", method_interceptor)
        test_obj = intercepted(3)
        self.assertEquals((3 + 5) * 4, test_obj.method(5))

    # TODO: refactor and split in two.
    def test_class_method_interception(self):
        def class_method_interceptor(context):
            value = context.proceed()
            return value * 2


        intercepted = intercept(self.MyTestClass, "class_method", class_method_interceptor)
        test_obj = intercepted
        self.assertEquals((5 + 12) * 2, test_obj.class_method(5))
        test_obj = intercepted(0)
        self.assertEquals((5 + 12) * 2, test_obj.class_method(5))


    # TODO: split test in two parts.
    def test_static_method_interception(self):
        def static_method_interceptor(context):
            value = context.proceed()
            return value * 2


        intercepted = intercept(self.MyTestClass, "static_method", static_method_interceptor)
        test_obj = intercepted
        self.assertEquals(( 5 + 7) *2, test_obj.static_method(5))
        test_obj = intercepted(0)
        self.assertEquals((5 + 7) * 2, test_obj.static_method(5))


    def test_instancemethod_interception_preserves_original_func_dict(self):
        def method_interceptor(context):
            return context.proceed()

        intercepted = intercept(self.MyTestClass, "method", method_interceptor)
        self.assertEquals(123, intercepted.method.someattribute)

    def test_classmethod_interception_preserves_original_func_dict(self):
        def method_interceptor(context):
            return context.proceed()

        intercepted = intercept(self.MyTestClass, "class_method", method_interceptor)
        self.assertEquals(456, intercepted.class_method.someattribute)

    def test_staticmethod_interception_preserves_original_func_dict(self):
        def method_interceptor(context):
            return context.proceed()

        intercepted = intercept(self.MyTestClass, "static_method", method_interceptor)
        self.assertEquals(789, intercepted.static_method.someattribute)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

from pydenji.config.pythonconfig import is_object_factory, dontconfigure
from pydenji.appcontext.aware import is_appcontext_aware

class NamingClashException(Exception):
    def __init__(self, clashing_attr, last_config):
        super(NamingClashException, self).__init__(
            "Name '%s' was configured multiple times, last in %s" % (
            clashing_attr, last_config)
        )

class CompositeConfig(object):
    def __init__(self, configs):
        # this resembles appcontext method.
        # TODO: check whether we could refactor it someway.
        # TODO: cyclomatic complexity is increasing dramatically in this method.
        # we'll need to split it into smaller pieces someday.
        for config in configs:
            for attr in dir(config):
                value = getattr(config, attr)
                if is_object_factory(value):
                    if not attr.startswith("_"):
                        # public factory, check for naming clashes.
                        if getattr(self, attr, None) is not None:
                            # TODO: it would probably be good to know
                            # where it was configured.
                            raise NamingClashException(attr, config)
                        setattr(self, attr, value)
                    else:
                        # private factory, prevent naming clashes.
                        # TODO: maybe we could just make them random or sequential?
                        setattr(self, "_pydenji__%s_%s" % (config.__class__.__name__, attr), value)

        # retain reference in order to set app context.
        self._pydenji__CONFIGURATIONS = configs

    @dontconfigure
    def set_app_context(self, context):
        for config in self._pydenji__CONFIGURATIONS:
            if is_appcontext_aware(config):
                    config.set_app_context(context)



    
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

import unittest
import sys

from pydenji.importer import NI, import_or_reload, get_by_fqdn

class  TestImporter(unittest.TestCase):
    def test_importer_imports_required_object(self):
        import os
        self.assertTrue(NI("os.unlink") is os.unlink)

class TestImportReload(unittest.TestCase):
    def setUp(self):
        if "pydenji.test.dummymodule" in sys.modules:
            sys.modules.pop("pydenji.test.dummymodule")

    def tearDown(self):
         if "pydenji.test.dummymodule" in sys.modules:
            sys.modules.pop("pydenji.test.dummymodule")

    def test_import_or_reload_imports_unimported_modules(self):
        import_or_reload("pydenji.test.dummymodule")
        self.assertTrue("pydenji.test.dummymodule" in sys.modules)

    def test_import_or_reload_reloads_imported_modules(self):
        import pydenji.test.dummymodule
        pydenji.test.dummymodule.sbirulabba = 1
        import_or_reload("pydenji.test.dummymodule")
        self.assertEquals(0, pydenji.test.dummymodule.sbirulabba)

    def test_import_or_reload_returns_imported_module(self):
        import pydenji.test.dummymodule
        self.assertEquals(pydenji.test.dummymodule, import_or_reload("pydenji.test.dummymodule"))

    def test_get_by_fqdn_fetches_object(self):
        self.assertEquals("0", get_by_fqdn("pydenji.test.dummymodule:sbirulabba.__str__")())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni


class SomeNetworkedClass(object):
    def __init__(self, connector, resource):
        self.connector = connector
        self.resource = resource

    def doSomething(self):
        self.connector.connect()


class SomeConnector(object):
    destination_filename_prefix = None
    
    def __init__(self, target_address):
        self.address = target_address

    def connect(self):
        # just mark it did something. # FIXME: Windows support?
        f = open(self.destination_filename_prefix + self.address, "w").write("something")
    


class SomeResource(object):
    pass

class SomeService(object):
    def __init__(self, NetworkedClassFactory):
        self.NetworkedClassFactory = NetworkedClassFactory

    def performAction(self):
        networkedclass = self.NetworkedClassFactory()
        networkedclass.doSomething()





#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase

from pydenji.config.pythonconfig import Configuration, prototype, singleton, dontconfigure
from pydenji.config.pythonconfig import is_object_factory, is_eager, _CONFIGURED_OBJECT_FACTORY

class TestConfig(TestCase):
    def test_config_configures_undecorated_public_methods(self):
        config_recorder = []
        class Simple(object):
            def public(self):
                pass
        self.conf = Configuration(Simple, config_recorder.append)
        self.assert_(Simple.public in config_recorder, "public was not configured.")

    def test_config_doesnt_configure_private_methods(self):
        class Simple(object):
            def _private(self):
                pass
        config_recorder = []
        self.conf = Configuration(Simple, config_recorder.append)
        self.assert_(not config_recorder)

    def test_config_doesnt_configure_dontconfigure_methods(self):
        class Simple(object):
            @dontconfigure
            def public(self):
                pass
        config_recorder = []
        self.conf = Configuration(Simple, config_recorder.append)
        self.assert_(not config_recorder)

    def test_config_doesnt_reconfigure_already_configured_object(self):
        class Simple(object):
            
            def already_configured(self):
                pass
            setattr(already_configured, _CONFIGURED_OBJECT_FACTORY, True)

        config_recorder = []
        self.conf = Configuration(Simple, config_recorder.append)
        self.assert_(not config_recorder)

    def test_prototype_factory_works_without_parameters(self):
        class Prototypal(object):
            @prototype
            def factory(self):
                return 1

        conf = Prototypal()
        conf.factory()

    def test_lazy_factory_works_without_parameters(self):
        class Lazy(object):
            @singleton.lazy
            def factory(self):
                return 1

        conf = Lazy()
        conf.factory()


class TestScopeDecorators(TestCase):
    def setUp(self):
        def iterator():
            yield 1
            yield 2
            yield 3

        self.func = iterator().send

    def test_prototype_creates_new_object_each_time(self):
        f = prototype(self.func)
        self.assertEquals([1,2,3], [f(None), f(None), f(None)])

    def test_prototype_configures_object(self):
        f = prototype(self.func)
        self.assertTrue(is_object_factory(f))

    def test_prototype_hints_for_lazy_instantiation(self):
        f = prototype(self.func)
        self.assertFalse(is_eager(f))

    def test_singleton_caches_first_returned_object(self):
        f = singleton(self.func)
        self.assertEquals([1,1,1], [f(None), f(None), f(None)])

    def test_singleton_configures_object(self):
        f = singleton(self.func)
        self.assertTrue(is_object_factory(f))

    def test_singleton_hints_for_eager_instantiation(self):
        f = singleton(self.func)
        self.assertTrue(is_eager(f))

    def test_lazy_singleton_caches_first_returned_object(self):
        f = singleton.lazy(self.func)
        self.assertEquals([1,1,1], [f(None), f(None), f(None)])

    def test_lazy_singleton_configures_object(self):
        f = singleton.lazy(self.func)
        self.assertTrue(is_object_factory(f))

    def test_lazy_singleton_hints_for_lazy_instantiation(self):
        f = singleton.lazy(self.func)
        self.assertFalse(is_eager(f))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

from pydenji.appcontext.aware import is_appcontext_aware
from pydenji.config.pythonconfig import Configuration
from pydenji.config.pythonconfig import singleton


def _set_app_context(self, context):
    self._pydenji__app_context = context

def _getattr(self, attr):
    # THIS IS BAD!
    try:
        return lambda *args, **kwargs: self._pydenji__app_context.get_object(attr, *args, **kwargs)
    except:
        # TODO: better error interception! just intercept what we
        # need to handle.
        raise KeyError, "'%s' object has no attribute '%s'" % (self, attr)

def ContextConfiguration(cls, configure_with=singleton, suffix=""):
    """
    Just like Configuration, but any unfound factory will be looked up in the app context.
    """
    
    ConfigClass = Configuration(cls, configure_with, suffix)
    # it might be appcontext aware but we
    # could not detect it without ABCs...
    # TODO: think about that.
    if is_appcontext_aware(ConfigClass):
        # just return it, it's already got whatever it needs. (maybe?)
        # TODO: let's think if this can do any harm.
        return ConfigClass

    configured_dict = {}

    configured_dict["set_app_context"] = _set_app_context
    configured_dict["_pydenji__app_context"] = None
    configured_dict["__getattr__"] = _getattr

    cls_type = type(cls)
    return cls_type(cls.__name__ + suffix, (ConfigClass, ), configured_dict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.
#
# Commandline integration for property mapping.


class CommandLinePropertyParser(object):
    def parse(self, arglist):
        return [ arg[3:] for arg in arglist if arg.startswith("-PD") ]





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

import unittest
from pydenji.userproperties.mapping import map_properties_to_obj, ConfigObjPropertyMapper
from pydenji.userproperties.mapping import inject_properties_from



class MockConfig2(object):
    def __init__(self, props):
        self.props = props

    def mymethod(self):
        some = self.props["missingkey"]

class TestConfigObjMappingTestCase(unittest.TestCase):
    example_config = """
    # for an object
    [object]
    property1="a"
    property2="b"

    # when are they set? automagically? explicitly? they can be both set?
    # probably: if property1 exists, then it is set, otherwise it isn't, and
    # everything is available in the config as well.

    # do I want to deliver something to a config itself?
    [SomeConfig]
    prop1="x"

    # placeholders resolution? there're no java properties here...
    # but my config REALLY needs something from outside.
    # special placeholder and if not resolved crash after instantiation?

    e.g. something like

    @Configuration
    class MyConfig(object):
        someprop = MustBeSet()

        ...

    


    """
    class TestObj(object):
        property1 = None


    def test_dictmapper_maps_existent_properties(self):
        o = self.TestObj()
        map_properties_to_obj({"property1": 5}, o)
        self.assertEquals(5, o.property1)


    def test_dictmapper_forbids_mapping_non_existent_properties_by_default(self):
        o = self.TestObj()
        self.assertRaises(ValueError, map_properties_to_obj,
            {"property2": 5}, o)

    def test_dictmapper_allows_mapping_non_existent_properties_if_asked(self):
        o = self.TestObj()
        map_properties_to_obj({"property2": 5}, o, map_nonexistent=True)
        self.assertEquals(5, o.property2)

    def test_mapping_properties_to_config(self):
        class MockConfig(object):
            property1 = 1

        
        config = ConfigObjPropertyMapper(
        ["[MockConfig]", "property1=123"]
        )(MockConfig)()
        self.assertEquals(123, config.property1)

    def test_properties_are_injected_as_keyword(self):
        class MockConfig1(object):
            def __init__(self, props):
                self.props = props
                
        config = inject_properties_from(["[MockConfig1]", "property1=123"])(MockConfig1)()
        self.assertEquals(123, config.props["property1"])

    def test_properties_raises_error_on_unset_properties(self):
        config_cls = inject_properties_from(["[MockConfig2]", "property1=123"])(MockConfig2)
        self.assertRaises(ValueError, config_cls)
        
        
    def test_later_co_sources_override_previous_values(self):
        class MockConfig1(object):
            def __init__(self, props):
                self.props = props

        config = inject_properties_from(["[MockConfig1]", "property1=123"], ["[MockConfig1]", "property1=456"])(MockConfig1)()
        self.assertEquals(456, config.props["property1"])

    def test_mapping_should_fail_if_section_name_is_just_a_property(self):
        class MockConfig1(object):
            def __init__(self, props):
                self.props = props

        self.assertRaises(TypeError, inject_properties_from(["MockConfig1=1"])(MockConfig1))

    def test_mapping_should_not_fail_if_config_name_missing(self):
        class MockConfig1(object):
            def __init__(self, props):
                self.props = props

        config = inject_properties_from(["property1=123"])(MockConfig1)()

    def test_global_property_injection(self):
        class MockConfig1(object):
            def __init__(self, props):
                self.props = props

        config = inject_properties_from(["[global]", "property1=123"])(MockConfig1)()
        self.assertEquals(123, config.props["property1"])

    def test_config_specific_properties_override_global_ones(self):
        class MockConfig1(object):
            def __init__(self, props):
                self.props = props

        config = inject_properties_from(["[global]", "property1=123", "[MockConfig1]",
                        "property1=456"])(MockConfig1)()
        self.assertEquals(456, config.props["property1"])
















#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.
from inspect import getabsfile
from compiler import parseFile
from configobj import ConfigObj, Section

from pydenji.userproperties.properties import UserProperties
from pydenji.userproperties.codescraper import get_getitem_accesses
from pydenji._aop.intercept import intercept


_NO_VALUE = object()

def _get_mapping_types():
    # this is not exactly nice...
    mapping_types = [dict]
    try:
        from collections import Mapping
        mapping_types.append(Mapping)
    except:
        pass

    from UserDict import UserDict, DictMixin
    mapping_types.append(UserDict)
    mapping_types.append(DictMixin)

    return tuple(mapping_types)


def map_properties_to_obj(d, obj, map_nonexistent=False):
    for key, value in d.iteritems():
        # TODO: this is a simple check, what if "value" is a method?
        if (getattr(obj, key, _NO_VALUE) is _NO_VALUE) and not map_nonexistent:
            raise ValueError, "Object '%s' hasn't got a '%s' attribute" % (obj, key)
        setattr(obj, key, value)
    return obj

#TODO: think about a real use case for this class :-/ we might just kill it.
class ConfigObjPropertyMapper(object):
    def __init__(self, configobj_source):
        self._co = ConfigObj(configobj_source, unrepr=True)

    def __call__(self, config_cls):
        # FIXME: we'll need a utility "aop" function to intercept calls.
        original_init = config_cls.__init__
        def new_init(new_self, *args, **kwargs):
            original_init(new_self, *args, **kwargs)
            # TODO: should we let the config name to be set explicitly?
            # TODO: name is changed by our decorator right now, maybe it shouldn't.
            map_properties_to_obj(self._co[config_cls.__name__], new_self)
            
        config_cls.__init__ = new_init
        return config_cls

# TODO: this might force property redefinition. We might think about
# a "global" namespace where each and every property is globally defined,
# and might be overriden later.
class inject_properties_from(object):
    def __init__(self, base_co_src, *other_co_srcs, **kwargs):
        self._co = ConfigObj(base_co_src, unrepr=True)
        for other_src in other_co_srcs:
            self._co.merge(ConfigObj(other_src, unrepr=True))
        self._target = kwargs.pop("target_kwarg", "props")
        
    def __call__(self, config_cls):
        original_init = config_cls.__init__
        def new_init(new_self, *args, **kwargs):
            # do we need to do something like an override?
            if self._target in kwargs:
                raise ValueError, "'%s' kw args was supplied already!"
            props = UserProperties(self._get_config_dict_for(config_cls.__name__))
            kwargs[self._target] = props
            original_init(new_self, *args, **kwargs)
            self._verify_no_missing_property(new_self, props, self._target)


        config_cls.__init__ = new_init
        return config_cls

    def _verify_no_missing_property(self, config_instance, props, attr):
        difference = set(get_getitem_accesses(config_instance, attr)).difference(set(props.keys()))
        if difference:
            raise ValueError, "Some property is missing: %s" % " ".join(difference)

    def _get_config_dict_for(self, section_name):
        # we want global_dict to be a ConfigObj section in order to support merging.
        global_dict = self._co.get("global", ConfigObj(["[global]"])["global"])
        if not isinstance(global_dict, Section):
            raise TypeError, "'%s' must be a section header" % "global"

        config_dict = self._co.get(section_name, {})
        if not isinstance(config_dict, _get_mapping_types()):
            raise TypeError, "'%s' must be a section header or dict." % section_name
        global_dict.merge(config_dict)
        return global_dict





        













#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from unittest import TestCase
import os
from pydenji.uriresolver import resource_filename_resolver



class TestFileResolver(TestCase):
    def test_resource_resolver_retrieves_filesystem_filename(self):
        resolved = resource_filename_resolver("file:/tmp/testfile")
        self.assertEquals("/tmp/testfile", resolved)

    def test_resolver_refuses_relative_file_resource(self):
        self.assertRaises(ValueError, resource_filename_resolver, "file:tmp/testfile")

    def test_uris_without_scheme_are_treated_as_file_uris(self):
        resolved = resource_filename_resolver("/tmp/testfile")
        self.assertEquals("/tmp/testfile", resolved)

class TestPackageResolver(TestCase):
    def test_resolver_retrieves_package_resource_filename(self):
        resolved = resource_filename_resolver("pkg://pydenji/test/test_uriresolver.py")
        # I don't want to use pkg_resources here because it seems not a proper
        # test just repeating what the actual implementation will be.
        # must strip Cs and Os just in case we're running from pyc or pyo.
        self.assertEquals(os.path.abspath(__file__.rstrip("co")), resolved)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from unittest import TestCase
from pydenji.pathtools import get_successive_path_pieces

class TestPathTools(TestCase):
    def test_absolute_file_pieces_are_returned(self):
        self.assertEquals(["/", "/path/", "/path/myfile"],
            get_successive_path_pieces("/path/myfile") )

    def test_absolute_dir_pieces_are_returned(self):
        self.assertEquals(["/", "/path/", "/path/mydir/"],
            get_successive_path_pieces("/path/mydir/") )

    def test_root_is_returned_as_such(self):
        self.assertEquals(["/"],
            get_successive_path_pieces("/") )

    def test_trailing_slash_retained_just_once(self):
        # if the path has a trailing slash it should be retained,
        # but just once, e.g. we don't want to return both path and path/
        self.assertEquals(["/", "/path/"],
            get_successive_path_pieces("/path/") )

    def test_relative_doesnt_work(self):
        self.assertRaises(IOError,
            get_successive_path_pieces, "relative/path" )

    def test_relative_works_if_forced_absolute(self):
        pieces = get_successive_path_pieces("path/myfile", force_absolute=True)
        # I don't like two assertions per test, but I wouldn't know how to
        # change this now without doing strange things.
        self.assertTrue(pieces[-1].endswith("path/myfile"))
        self.assertTrue(pieces[-2].endswith("path/"))







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

# very rough and basic integration test. To be extended.

import os
from unittest import TestCase

from pydenji.test.integration.simple_app import *

from pydenji.appcontext.wiring import ArgNameContextWirer
from pydenji.config.contextconfig import ContextConfiguration

from pydenji.appcontext.context import AppContext
from pydenji.userproperties.mapping import inject_properties_from
from pydenji.userproperties.overrider import override_with
from pydenji.config.pythonconfig import Configuration, prototype, singleton, dontconfigure
from pydenji.uriresolver import resource_filename_resolver as rfr
from pydenji.placeholders import Placeholder


@inject_properties_from(rfr("pkg://pydenji/test/integration/resources/inject.conf"))
@override_with(rfr("pkg://pydenji/test/integration/resources/propertyconf.properties"))
@Configuration
class MyRemoteFetchService(object):
    target_address = Placeholder("target_address")

    def __init__(self, props):
        self.target_address = props["target_address"]
        self.props = props

    @dontconfigure
    def set_app_context(self, context):
        self.wirer = ArgNameContextWirer(context, self.props)
    
    def network_service(self):
        return self.wirer.wire(SomeService)


    @prototype
    def NetworkedClassFactory(self):
        return self.wirer.wire(SomeNetworkedClass)

    @prototype
    def connector(self):
        return self.wirer.wire(SomeConnector)

    @singleton.lazy
    def resource(self):
        return SomeResource()

class DisableTestSimpleConfiguration(TestCase):
    def setUp(self):
        try:
            os.unlink("/tmp/pydenji_simple_configuration_test_somenetworkaddress")
        except:
            pass

    def disable_test_network_service_fetching(self):
        context = AppContext(MyRemoteFetchService())
        network_service = context.get_object("network_service")
        network_service.performAction()
        # the file should be created by connector instance, whose prefix was overriden.
        self.assertTrue(os.path.exists("/tmp/pydenji_simple_configuration_test_somenetworkaddress"), "missing file it should be created")
        os.unlink("/tmp/pydenji_simple_configuration_test_somenetworkaddress")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from functools import partial
from types import UnboundMethodType

from pydenji.appcontext.aware import is_appcontext_aware
from pydenji._aop.intercept import intercept
from pydenji.placeholders import Placeholder

# TODO: we might probably reduce the number of constants.
_CONFIGURED_OBJECT_FACTORY = "_pydenji__CONFIGURED_OBJECT_FACTORY"
_INSTANTIATE_EAGERLY = "_pydenji__INSTANTIATE_EAGERLY"
_SHOULD_CONFIGURE = "_pydenji__SHOULD_CONFIGURE"

def is_object_factory(obj):
    if getattr(obj, _CONFIGURED_OBJECT_FACTORY, None) is True:
        return True
    return False

def is_eager(obj):
    if getattr(obj, _INSTANTIATE_EAGERLY, None) is True:
        return True
    return False

def should_be_configured(obj):
    return getattr(obj, _SHOULD_CONFIGURE, True)

class _Maybe(object):
    def __init__(self, has_value=False, value=None):
        self.has_value = has_value
        self.value = value

def singleton(func, eager=True):
    maybevalue = _Maybe()
    def singleton_wrapped(self, *args, **kwargs):
        if args or kwargs:
            raise TypeError, "Singleton mustn't take any parameter. Use per-instance config instead."

        if not maybevalue.has_value:
            maybevalue.value = func(self)
            maybevalue.has_value = True
        return maybevalue.value

    setattr(singleton_wrapped, _INSTANTIATE_EAGERLY, eager)
    setattr(singleton_wrapped, _CONFIGURED_OBJECT_FACTORY, True)
    setattr(singleton_wrapped, _SHOULD_CONFIGURE, False)

    return singleton_wrapped

singleton.lazy = partial(singleton, eager=False)

def prototype(func):
    def f(*args, **kwargs):
        return func(*args, **kwargs)
    setattr(f, _CONFIGURED_OBJECT_FACTORY, True)
    setattr(f, _INSTANTIATE_EAGERLY, False)
    setattr(f, _SHOULD_CONFIGURE, False)
    return f

def dontconfigure(func):
    def f(*args, **kwargs):
        return func(*args, **kwargs)
    setattr(f, _SHOULD_CONFIGURE, False)
    setattr(f, _INSTANTIATE_EAGERLY, False)
    setattr(f, _CONFIGURED_OBJECT_FACTORY, False)
    return f


def _to_be_configured(clsattr, attrvalue):
    return ((not clsattr.startswith("_")) and
        isinstance(attrvalue, UnboundMethodType) and
        not is_object_factory(attrvalue) and
        should_be_configured(attrvalue))

# TODO: this function means nothing, rename it.
def _configure_with(cls, configure_with):
    configured_dict = {}
    for clsattr in dir(cls):
        attrvalue = getattr(cls, clsattr)
        if _to_be_configured(clsattr, attrvalue):
            configured_dict[clsattr] = configure_with(attrvalue)
    return configured_dict

def Configuration(cls, configure_with=singleton, suffix=""):
    """
    Makes all public, unwrapped methods *eager singletons* by default.
    Also, after instantiation a "params" instance attribute will be set -
    it will hold a dictionary.

    Also sets required_properties init interceptor; if the class or the
    instance has got a "required_properties" attribute, after initialization
    that "resolve" required properties by settings placeholders instead.

    (this won't completely solve missing placeholder configuration, by the way).
    

    Non-public methods and already-wrapped methods will just go untouched.
    """
    configured_dict = _configure_with(cls, configure_with)
    cls_type = type(cls)
    return cls_type(cls.__name__ + suffix, (cls, ), configured_dict)











# To change this template, choose Tools | Templates
# and open the template in the editor.

import unittest

from pydenji.appcontext.wiring import ArgNameContextWirer
from pydenji.appcontext.wiring import PropertyAndContextMergingProxy
from pydenji.appcontext.wiring import NameClashError

class Peer(object):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def __repr__(self):
        return "<Peer %s %s>" % (str(self.args), str(self.kwargs))

    def __eq__(self, other):
        return True if (
                (other.args == self.args) and
                (other.kwargs == self.kwargs)
                ) else False
                

class DummyContext(object):
    def get_object(self, name, *args, **kwargs):
        if name != "peer":
            raise KeyError, "Can't handle anything different from peer"
        return Peer(*args, **kwargs)

    def __contains__(self, key):
        return True if (key == "peer") else False

    def __iter__(self):
        return iter(["peer"])
    
def factory(a, b, peer):
    return (a, b, peer)

class TestContextWiring(unittest.TestCase):
    def test_contextwiring_fetches_data_from_mapping_and_context(self):
        context = DummyContext()
        mapping = {"a":1, "b":2}
        cw = ArgNameContextWirer(context, mapping)
        wired = cw.wire(factory)
        self.assertEquals((1, 2, Peer()), wired)

    def test_call_time_wiring_args_can_override_mapping_values(self):
        context = DummyContext()
        mapping = {"a":1, "b":2}
        cw = ArgNameContextWirer(context, mapping)
        wired = cw.wire(factory, 5, b=4)
        self.assertEquals((5, 4, Peer()), wired)
    
    def test_call_time_wiring_args_can_override_context_values(self):
        context = DummyContext()
        mapping = {"a":1, "b":2}
        cw = ArgNameContextWirer(context, mapping)
        wired = cw.wire(factory, peer="foo")
        self.assertEquals((1, 2, "foo"), wired)


class TestPropertyAndContextMergingProxy(unittest.TestCase):
    def setUp(self):
        context = DummyContext()
        mapping = {"a":3, "b":4}
        self.pacmp = PropertyAndContextMergingProxy(context, mapping)

    def test_mergingproxy_fetches_value_from_mapping_if_unavailable_in_context(self):
        self.assertEquals(3, self.pacmp["a"])

    def test_mergingproxy_fetches_value_from_context_if_unavailable_in_mapping(self):
        self.assertEquals(Peer(), self.pacmp["peer"])

    def test_mergingproxy_prohibits_proxying_if_clashing_values(self):
        context = DummyContext()
        mapping = {"peer":3, "b":4}
        self.assertRaises(NameClashError, PropertyAndContextMergingProxy, context, mapping)


if __name__ == '__main__':
    unittest.main()




import os.path
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

# check:
# how to work with pkg_resources Requirement classes
# how to work with pkg_resources split packages, e.g. those using namespace_packages
# how will this play along distutils2 and its pkgutil.open()
# support unicode/encodings/etc.

import os

from pydenji.pathtools import verify_path_existence
from pydenji.uriresolver import resource_filename_resolver

class ResourceAccessError(IOError):
    pass

class RWResource(object):
    _resolver = staticmethod(resource_filename_resolver)

    filename = None
    _mode = None
    _buffering = None
    _opened = False
    _file_obj = None


    def __init__(self, uri, mode, buffering, opener=open):
        # this attr is kept for backward compatibility, but will be removed.
        self.filename = self._resolver(uri)
        self._mode = mode
        self._buffering = buffering
        self._opener = opener
        self._verify_consistency()

    def _verify_consistency(self):
        # template method, should raise an exception
        # if something is wrong.
        pass

    def __getattr__(self, attr):
        if not self._opened:
            self._open()
        return getattr(self._file_obj, attr)

    def _open(self):
        self._file_obj = self._opener(self.filename, self._mode, self._buffering)
        self._opened = True

    @property
    def mode(self):
        return self._mode

    @property
    def name(self):
        return self.filename


class ReadResource(RWResource):
    def __init__(self, uri, binary=False):
        super(ReadResource, self).__init__(
            uri, "r" + ("b" if binary else ""), -1)

    def _verify_consistency(self):
        verify_path_existence(self.filename, ResourceAccessError)
        if not os.access(self.filename, os.R_OK):
            raise ResourceAccessError, ("Insufficient privileges, "
                "can't read '%s' " % self.filename)

class WriteResource(RWResource):
    def _verify_consistency(self):
        write_path_dir, basename = os.path.split(self.filename)
        verify_path_existence(write_path_dir, ResourceAccessError)
        if not os.path.isdir(write_path_dir):
            raise ResourceAccessError, "'%s' is not a directory, can't write '%s'" % (
                write_path_dir, basename)
        if not os.access(write_path_dir, os.W_OK):
            raise ResourceAccessError, ("Insufficient privileges, "
                "can't write '%s' in '%s' " % (basename, write_path_dir))
        if os.path.exists(self.filename) and not os.access(self.filename, os.W_OK):
            raise ResourceAccessError, ("Insufficient privileges, can't overwrite '%s'" %
                self.filename)

def OverwritingWriteResource(uri, binary=False):
    return WriteResource(uri, "w" + ("b" if binary else ""), -1)

def AppendingWriteResource(uri, binary=False):
    # TODO: do we need an appender which just appends, e.g. never creates?
    return WriteResource(uri, "a" + ("b" if binary else ""), -1)

class NewFileWriteResource(WriteResource):
    def __init__(self, uri, binary=False):
        super(NewFileWriteResource, self).__init__(uri, "w" + ("b" if binary else ""), -1)
        
    def _verify_consistency(self):
        if os.path.exists(self.filename):
            raise ResourceAccessError, "'%s' already exists." % self.filename
        super(NewFileWriteResource, self)._verify_consistency()

class ExecutableResource(object):
    _resolver = staticmethod(resource_filename_resolver)

    def __init__(self, uri):
        self.filename = self._resolver(uri)
        self.name = self.filename
        self._verify_consistency()

    def _verify_consistency(self):
        verify_path_existence(self.filename, ResourceAccessError)
        if not os.access(self.filename, os.X_OK):
            raise ResourceAccessError, ("Insufficient privileges, "
                "can't execute '%s' " % self.filename)

def enumerateResources(resource, childrenResourceFactory=ReadResource):
    # this is a bit naive, but we probably won't need anything more.
    if os.path.isdir(resource.filename):
        return [childrenResourceFactory(resource.filename + os.sep + path) for
            path in os.listdir(resource.filename)]
    else:
        return [childrenResourceFactory(resource.filename)]
            


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from byteplay import Code
from byteplay import LOAD_FAST
from byteplay import LOAD_ATTR
from byteplay import LOAD_CONST
from byteplay import BINARY_SUBSCR
from byteplay import STORE_FAST


import inspect

_ANY_VALUE = object()


class _MatchingTuple(tuple):
    def __eq__(self, other):
        for s, o in zip(self, other):
            if (s is not _ANY_VALUE) and (s != o):
                return False
        return True

# variation on stackoverflow #3847386 by Dave Kirby
def _accumulate_contained(small, big):
    accumulator = []
    for i in xrange(len(big) - len(small)+1):
        for j in xrange(len(small)):
            if small[j] == big[i+j]:
                pass
            else:
                break
        else:
            accumulator.append(i)
    return accumulator


def _get_accumulated(func, usage_matcher, target_offset):
    # what should we name this func??
        accumulator = []
        disassembled_code = Code.from_code(func.func_code).code
        contained = _accumulate_contained(usage_matcher, disassembled_code)
        for contained_index in contained:
            accumulator.append(disassembled_code[contained_index + target_offset][1])
        return accumulator

def get_getitem_accesses(obj, attr):
    """
    For whatever member function of obj, return a set containing all the keys
    for which obj.attr is accessed through __getitem__.
    """
    usage_matcher = [(LOAD_FAST, 'self'), (LOAD_ATTR, attr),
        _MatchingTuple((LOAD_CONST, _ANY_VALUE)), (BINARY_SUBSCR, None)]
    funcs = inspect.getmembers(obj, inspect.ismethod)

    used_keys = set()

    # directly accessed self.attr __getitem__
    for name, func in funcs:
        used_keys.update(_get_accumulated(func, usage_matcher, 2))

    # accessed after saving props somewhere
    usage_matcher = [(LOAD_FAST, 'self'), (LOAD_ATTR, attr),
                _MatchingTuple((STORE_FAST, _ANY_VALUE))]
    
    for name, func in funcs:
        stored_attributes = _get_accumulated(func, usage_matcher, 2)
        for stored_attr in stored_attributes:
            attr_usage_matcher = [(LOAD_FAST, stored_attr),
                _MatchingTuple((LOAD_CONST, _ANY_VALUE)), (BINARY_SUBSCR, None)]
            used_keys.update(_get_accumulated(func, attr_usage_matcher, 1))

    return used_keys

    









#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

import unittest

from pydenji.userproperties.overrider import override_with

class Obj(object):
    pass

class OverridenConfig(object):
    def myobj(self):
        myobj = Obj()
        myobj.property1 = 123
        return myobj

class TestPropertyOverrider(unittest.TestCase):
    def test_property_overriding(self):
        config = override_with(["[myobj]", "property1=456"])(OverridenConfig)()
        self.assertEquals(456, config.myobj().property1)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

# todo: handle non-str paths.
import os
import time


# lot of nesting here... can we reduce cyclomatic complexity of this code?
def _appender(pieces, piece):
    if pieces:
        if (pieces[-1].rstrip(os.sep) == piece):
        # don't append /path if /path/ is there.
            return
        if not piece.endswith(os.sep):
            # if there're already other pieces this piece must
            # be a dir; force it with a trailing slash.
            piece += os.sep
    pieces.append(piece)

def get_successive_path_pieces(path, force_absolute=False):
    # this will just support ABSOLUTE paths right now, either
    # passed in as such or forced.
    # think about relative path support.
    pieces = []

    if force_absolute:
        path = os.path.abspath(path)

    if not path.startswith(os.sep):
        raise IOError, "Path '%s' is not absolute and was not forced as such." % path

    parent = path

    while True:
        parent, base = os.path.split(parent)
        if (parent == os.sep):
            piece = parent + base
            _appender(pieces, piece)
            if base == "":
                break
        else:
            # if the former piece is equivalent, just with a trailing slash,
            # don't add another piece.
            piece = parent + os.sep + base
            _appender(pieces, piece)
   

    pieces.reverse()
    return pieces

class NotExistingPath(IOError):
    pass

def verify_path_existence(full_path, on_error_raise=NotExistingPath):
    """
    Verifies that full_path exists. If it doesn't, it throws a proper and clear error that
    helps understanding what's the issue with such path.
    """
    
    # quick bailout
    if os.path.exists(full_path):
        return

    # check why it does not exist.
    path_pieces = get_successive_path_pieces(full_path)

    for piece in path_pieces[:-1]:
        if not os.path.exists(piece):
            raise on_error_raise, "Directory '%s' does not exist while looking for '%s'." % (
                piece, full_path)
        if not os.path.isdir(piece):
            raise on_error_raise, "'%s' is not a directory while looking for '%s'" % (piece,
                full_path)
        if not os.access(piece, os.X_OK):
            raise on_error_raise, "Traversal (x) permission denied on '%s', can't determine '%s' existence" % (
                piece, full_path)

    if not os.path.exists(full_path):
        raise on_error_raise, "'%s' does not exist." % full_path

    # if we get here something is wrong.
    raise AssertionError, "Something is wrong with this function."





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.
from configobj import ConfigObj
from pydenji._aop.intercept import intercept

# TODO: change this name, I don't like it.
class override_with(object):
    def __init__(self, configobj_source, *other_co_srcs):
        self._co = ConfigObj(configobj_source, unrepr=True)
        for other_src in other_co_srcs:
            self._co.merge(ConfigObj(other_src, unrepr=True))

    def __call__(self, config_cls):
        for section_name in self._co.sections:
            def section_interceptor(context):
                o = context.proceed()
                for k, v in self._co[section_name].items():
                    setattr(o, k, v)
                return o
            # this creates a new subclass every time! we should change the way
            # intercept works??
            config_cls = intercept(config_cls, section_name, section_interceptor)

        return config_cls



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

# TODO: add unit tests for this.

class UnconfiguredError(Exception):
    def __init__(self, name):
        Exception.__init__(self, "'%s' placeholder has not been configured!" % name)

class Placeholder(object):
    def __init__(self, name):
        object.__setattr__(self, "name", name)

    def __getattr__(self, attr):
        raise UnconfiguredError, self.name

    def __setattr__(self, attr, value):
        raise UnconfiguredError, self.name


PH = Placeholder # quick alias



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from urlparse import urlparse, uses_netloc
from pkg_resources import resource_filename

#TODO: remove this sort of "static initializer"
if "pkg" not in uses_netloc:
    uses_netloc.append("pkg")

def file_uri_resolver(parsed_uri):
    if not parsed_uri.path.startswith("/"):
        raise ValueError, "Relative paths are not supported."
    if parsed_uri.netloc:
        raise ValueError, "Netloc in file scheme is unsupported."
    return parsed_uri.path

def package_uri_resolver(parsed_uri):
    return resource_filename(parsed_uri.netloc, parsed_uri.path)

supported_schemes = {
    "file" : file_uri_resolver,
    "" : file_uri_resolver,
    "pkg": package_uri_resolver,
}

def resource_filename_resolver(resource_uri):
    parsed = urlparse(resource_uri)
    if parsed.scheme not in supported_schemes:
        raise TypeError, "Scheme '%s' is unsupported" % parsed.scheme
    return supported_schemes[parsed.scheme](parsed)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

import unittest

from pydenji.config.argwiring import wire

def func_to_wire(pos1, pos2, kw1="a", kw2="b"):
    return (pos1, pos2, kw1, kw2)

class TestArgwiring(unittest.TestCase):
    def test_all_nonvariable_arguments_can_be_wired_from_dictionary(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3, "kw2": 4 }
        obj = wire(func_to_wire, d)
        self.assertEquals( (1,2,3,4), obj )

    def test_missing_args_from_mapping_are_fetched_from_defaults(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3 }
        obj = wire(func_to_wire, d)
        self.assertEquals( (1,2,3,"b"), obj )

    def test_error_raised_if_required_args_cant_be_completed(self):
        d = { "pos1":1, }
        self.assertRaises(TypeError, wire, func_to_wire, d )

    def test_positional_additional_arguments_have_precedence(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3 }
        obj = wire(func_to_wire, d, "x")
        self.assertEquals( ("x", 2 ,3,"b"), obj)

    def test_positional_additional_kw_arguments_have_precedence(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3 }
        obj = wire(func_to_wire, d, pos2="y")
        self.assertEquals( (1, "y" ,3, "b"), obj)

    def test_positional_and_kw_arguments_dont_clash(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3 }
        obj = wire(func_to_wire, d, 5, pos2="y")
        self.assertEquals( (5, "y" ,3, "b"), obj)

    def test_overlapping_args_and_kwargs_raise_error(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3 }
        self.assertRaises(TypeError, wire, func_to_wire, d, 5, pos1="5")

    def test_too_many_args_raise_typeerror(self):
        d = { "pos1":1, "pos2": 2, "kw1": 3 }
        self.assertRaises(TypeError, wire, func_to_wire, d, 1,2,3,4,5)

    def test_argument_positions_do_not_get_mismatched(self):
        self.assertRaises(TypeError, wire, func_to_wire, {}, 1, kw1=1, kw2=4)

    def test_arguments_in_excess_do_not_interfer_with_instantiation(self):
        d = { "some":1, "argument": 2}
        obj = wire(func_to_wire, d, 1, 2)


def varposargs_func(arg1, *args):
    return (arg1, args)

def varkwargs_func(arg1, **kwargs):
    return (arg1, kwargs)

def varboth_func(*args, **kwargs):
    return (args, kwargs)

class TestVarArgsWiring(unittest.TestCase):
    def test_varargs_are_forwarded(self):
        obj = wire(varposargs_func, {}, 1, 2, 3)
        self.assertEquals((1, (2, 3)), obj)

    def test_varkw_are_forwarded(self):
        obj = wire(varkwargs_func, {"a": 5}, 1, a=2, b=3)
        self.assertEquals((1, {"a":2, "b":3}), obj)

    def test_varargs_and_varkw_are_forwarded(self):
        obj = wire(varboth_func, {}, 1, a=2, b=3)
        self.assertEquals(((1,), {"a":2, "b":3}), obj)



        
if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

from pydenji.resourceloader import ResourceAccessError
from pydenji.config.pythonconfig import Configuration
from pydenji.userproperties.mapping import inject_properties_from
from pydenji.userproperties.overrider import override_with
from pydenji.resourceloader import ReadResource, ResourceAccessError

# this is a quickly baked class which is highly prone to change in the future.
# don't rely on its API.

class QuickConfig(object):
    def __init__(self, prop_urls=(), override_urls=(), ignore_missing=True):
        self._prop_urls = prop_urls
        self._override_urls = override_urls
        self._ignore_missing = ignore_missing

    def __call__(self, config_cls):
        return self._get_injector()(self._get_overrider()(Configuration(config_cls)))

    def _get_injector(self):
        prop_resources = []
        for prop_url in self._prop_urls:
            try:
                resource = ReadResource(prop_url)
                prop_resources.append(resource)
            except ResourceAccessError:
                if not self._ignore_missing:
                    raise
        if not prop_resources:
            return lambda x: x
        return inject_properties_from(*prop_resources)

    def _get_overrider(self):
        override_resources = []
        for override_url in self._override_urls:
            try:
                resource = ReadResource(override_url)
                override_resources.append(resource)
            except ResourceAccessError:
                if not self._ignore_missing:
                    raise
        if not override_resources:
            return lambda x: x
        return override_with(*override_resources)
        

            





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

from unittest import TestCase

from pydenji.appcontext.context import AppContext
from pydenji.appcontext.aware import is_appcontext_aware
from pydenji.config.contextconfig import ContextConfiguration
from pydenji.config.pythonconfig import singleton, dontconfigure
from pydenji.config.pythonconfig import Configuration

class TestAppContext(TestCase):
    def test_appcontext_allows_retrieving_by_name(self):

        class MockConf(object):
            @singleton
            def something(self):
                return 1
        MockConf = Configuration(MockConf)
        
        context = AppContext(MockConf())
        something = context.get_object("something")
        self.assertEquals(1, something)

    def test_appcontext_supports_multiple_configs(self):
        # TODO: this functionality might be overlapping to CompositeConfig.
        # think about it.
        
        
        class MockConf(object):
            @singleton
            def something(self):
                return 1
        MockConf = Configuration(MockConf)

        class OtherConf(object):
            @singleton
            def otherthing(self):
                return 2

        OtherConf = Configuration(OtherConf)

        context = AppContext(MockConf(), OtherConf())
        something = context.get_object("something")
        self.assertEquals(1, something)
        otherthing = context.get_object("otherthing")
        self.assertEquals(2, otherthing)



    def test_appcontext_fetches_objects_eagerly_when_required(self):
        c = set()

        class MockConf(object):
            @singleton
            def something(self):
                c.add("something")

            @singleton
            def _private(self):
                c.add("private")

        MockConf = Configuration(MockConf)
             

        conf = MockConf()
        context = AppContext(conf)
        self.assertEquals(set(["something", "private"]), c)

        
    def test_appcontext_fetches_objects_lazily_when_required(self):
        c = []

        class MockConf(object):
            @singleton.lazy
            def something(self):
                c.append(True)
        MockConf = Configuration(MockConf)


        conf = MockConf()
        context = AppContext(conf)
        self.assertEquals([], c)

    def test_appcontext_gets_injected_on_aware_objects(self):
        # TODO: think whether we need to use an ABC instead or as well.
        class AppAwareObject(object):
            app_context = None
            
            def set_app_context(self, context):
                self.app_context = context


        class MockConf(object):
            @singleton
            def appcontextaware(self):
                return AppAwareObject()
        MockConf = Configuration(MockConf)

        context = AppContext(MockConf())
        aware = context.get_object("appcontextaware")
        self.assertTrue(context is aware.app_context, "context wasn't injected correctly!")

    def test_appcontext_gets_injected_on_aware_configuration_objects(self):

        class MockConf(object):
            @dontconfigure
            def set_app_context(self, context):
                self.app_context = context
        
        MockConf = Configuration(MockConf)

        conf = MockConf()
        context = AppContext(conf)
        self.assertTrue(context is conf.app_context, "context wasn't injected correctly!")


class TestGlobalConfig(TestCase):
    def test_global_config_falls_back_on_appcontext_factories(self):
        
        class One(object):
            def relies_on_other(self):
                return self.other() * 2
            
        class Other(object):
            def other(self):
                return 2

        one = ContextConfiguration(One)()
        other = Configuration(Other)()

        context = AppContext(one, other)
        self.assertEquals(4, context.get_object("relies_on_other"))


    






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from unittest import TestCase

from pydenji.userproperties.cmdline import CommandLinePropertyParser

class TestCmdlineIntegration(TestCase):
    def test_cmdline_arguments_starting_with_PD_are_parsed(self):
        parser =  CommandLinePropertyParser()
        # we'd like to feed those to configparser?
        self.assertEquals(["somevalue=1"], parser.parse(["-PDsomevalue=1" , "--someother"]))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from unittest import TestCase

from pydenji.config.composite import CompositeConfig, NamingClashException
from pydenji.config.pythonconfig import prototype, singleton

class FirstConfig(object):
    @singleton
    def first(self):
        return 1

class SecondConfig(object):
    @prototype
    def second(self):
        return 2

class SecondBisConfig(object):
    @singleton
    def second(self):
        return 3

    @prototype
    def _private(self):
        return 1

class TestCompositeConfig(TestCase):

    def test_composite_config_merges_input_configs(self):
        first = FirstConfig()
        second = SecondConfig()
        
        composite = CompositeConfig((first, second))

        self.assertEquals(1, composite.first())
        self.assertEquals(2, composite.second())

    def test_composite_config_raises_error_if_name_clash_occurs(self):
        first = SecondConfig()
        second = SecondBisConfig()

        self.assertRaises(NamingClashException, CompositeConfig, (first, second))

    def test_composite_config_doesnt_expose_private_factories(self):
        first = FirstConfig()
        second = SecondBisConfig()

        composite = CompositeConfig((first, second))

        self.assertRaises(AttributeError, getattr, composite, "_private")


